#ifndef __simulator_hxx__

#define __simulator_hxx__




//****************************************************************//
//*                                                              *//
//*------------------- Includes SECTION -------------------------*//
//*                                                              *//
//****************************************************************//

#include <queue>

#include "lois.hxx"


//****************************************************************//
//*                                                              *//
//------------ Public Data Structures SECTION --------------------//
//*                                                              *//
//****************************************************************//

using namespace std;





//****************************************************************//
//*                                                              *//
//----------------- Public Classes SECTION -----------------------//
//*                                                              *//
//****************************************************************//


//----------------------------------------------------------------//
// Class :    Event                                               //
//                                                                //
//                                                                //
// Description : classe representant un event M/M/1               //
//                                                                //
//----------------------------------------------------------------//

class Event
{
public:
  enum EventType { ARRIVAL=1, DEPARTURE, SAVE, END};

private:

  double     date_;
  EventType  type_;

public:

  //----------------------------------------------------------------//
  //                 CONSTRUCTEURS ET DESTRUCTEUR                   //
  //----------------------------------------------------------------//

  ///Constructeur par defaut
  Event( double t=0.0, EventType e=ARRIVAL):
    date_(t),
    type_(e)
  {}


  ///Constructeur par recopie
  Event( Event const &other ):
    date_(other.date_),
    type_(other.type_)
  {}

  
  //----------------------------------------------------------------//
  //                   ACCESSEURS                                   //
  //----------------------------------------------------------------//

  double getDate() { return date_; }

  EventType getType() { return type_; }


  //----------------------------------------------------------------//
  //                   ORDRE                                        //
  // Un evenement est d'autant plus prioritaire que sa date est     //
  // petite (inversion liee au priotity_queue)                      //
  //----------------------------------------------------------------//
  
  bool operator < (Event const &right) const {
    return ( date_ > right.date_ );
  }

};





//----------------------------------------------------------------//
// Class :    Simulator                                           //
//                                                                //
//                                                                //
// Description : classe representant un simulateur M/M/1          //
//                                                                //
//----------------------------------------------------------------//


class Simulator
{
  //attributs
private:

  double                  maxTime_;        //temps maximum de simulation          
  double                  delta_;          //Pas de temps de sauvegarde  de l'etat
  int                     nbSteps_;        //Nombre d'echantillons sauvegardes    
  int                     maxSamples_;     //Nombre maximum de trajectoires       

  ExpoGen               * arrivalGen_;     //Generateur des durees inter-arrivees 
  ExpoGen               * serviceGen_;     //Generateur des durees de service     
  priority_queue<Event>   evtList_;        //Echeancier                           

  double                  t;               //la date courante de simulation       
  int                     sample_;         //le numero de trajectoire courante    
  int                     X;               //le nombre de clients dans le systeme 

  double                  mean_;           //moyenne des trajectoires             
  double                  var_;            //variance des trajectoires            
  double                * trajectory_;     //trajectoire moyenne                  
  double                  stationaryTime_; //date de debut du stationnaire        

public:

  //----------------------------------------------------------------//
  //                 CONSTRUCTEURS ET DESTRUCTEUR                   //
  //----------------------------------------------------------------//

  ///Constructeur par defaut
  Simulator( double T=100.0, double dt=0.1, int N=100,
	     double lambda=0.5, double mu=1.0,
	     double steadyStateStart=50.0) :
    maxTime_(T), 
    delta_(dt),
    maxSamples_(N),
    stationaryTime_(steadyStateStart)
  {
    arrivalGen_ = new ExpoGen(1.0/lambda);
    serviceGen_ = new ExpoGen(1.0/mu);
    nbSteps_ = ((int) floor(T/dt))+1;
    trajectory_ = new double[ nbSteps_ ];
    for (int i=0; i<nbSteps_; i++) trajectory_[i] = 0.0;
    t = 0.0;
    sample_ = 0;
    mean_ = 0.0;
    var_ = 0.0;
  }

  // destructeur
  ~Simulator()
  {
    delete arrivalGen_;
    delete serviceGen_;
    delete [] trajectory_;
  }


  //----------------------------------------------------------------//
  //                   ACCESSEURS                                   //
  //----------------------------------------------------------------//

  double  getTime() { return t; }

  double  getMaxTime() { return maxTime_; }

  int     getSample() { return sample_; }

  int     getMaxSamples() { return maxSamples_; }


  //----------------------------------------------------------------//
  //               METHODES DE CALCUL                               //
  //----------------------------------------------------------------//


  double  nextArrivalTime() { return arrivalGen_->randomNumber() + t; }

  double  nextDepartureTime() { return serviceGen_->randomNumber() + t; }

  Event   nextEvent();

  void   addEvent(Event e);

  void reset();

  void run();

  void printTrajectory(char * fileName);

  void printStationaryParameters( );

};




#endif
